"""Context parser that returns a dictionary from a json string."""

import logging
import json

# use pypyrlogger to ensure loglevel is set correctly
logger = logging.getLogger(__name__)


def get_parsed_context(args):
    """Parse input context args and returns context as dictionary."""
    if not args:
        logger.debug("pipeline invoked without context arg set. For "
                     "this json parser you're looking for something "
                     "like: "
                     "pypyr pipelinename '{\"key1\":\"value1\","
                     "\"key2\":\"value2\"}'")
        return None

    logger.debug("starting")
    # deserialize the input context string into json
    return json.loads(' '.join(args))



"""Utility functions for evaluating expressions."""


def eval_string(input_string, locals):
    """Dynamically evaluates the input_string expression.

    This provides dynamic python eval of an input expression. The return is
    whatever the result of the expression is.

    Use with caution: since input_string executes any arbitrary code object,
    the potential for damage is great.

    For reasons best known to eval(), it EAFPs the locals() look-up 1st and if
    it raises a KeyNotFound error, moves on to globals.

    This means if you pass in something like the pypyr context, the custom
    KeyNotInContextError the pypyr context raises isn't caught, and thus eval
    doesn't work.

    Therefore, in order to be used as the locals arg, the pypyr context needs
    to be initialized to a new dict like this:
        out = eval_string('1==1', dict(context))

    Args:
        input_string: str. expression to evaluate.
        locals: dict-like. Mapping object containing scope for eval.

    Returns:
        Whatever object results from the string expression valuation.

    """
    # empty globals arg will append __builtins__ by default
    return eval(input_string, {}, locals)



"""Utility functions for type checking, conversion and handling."""


def are_all_this_type(type, *objects):
    """Return True if all objects are instances of this type.

    Args:
        type: Type to check against, e.g str, dict
        objects: *args of objects. Each of these will be checked against type.

    Returns:
        True is all objects are instances of specified Type.
        False if any or all of the objects are not instances of Type.

    """
    # return true if all objects are instances of type
    return all(isinstance(object, type) for object in objects)


def cast_to_type(obj, out_type):
    """Cast obj to out_type if it's not out_type already.

    If the obj happens to be out_type already, it just returns obj as is.

    Args:
        obj: input object
        out_type: type.

    Returns:
        obj cast to out_type. Usual python conversion / casting rules apply.

    """
    in_type = type(obj)
    if out_type is in_type:
        # no need to cast.
        return obj
    else:
        return out_type(obj)
